<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class WFACP_Compatibility_With_Divi {

	public function __construct() {

		add_action( 'after_setup_theme', function () {
			if ( isset( $_GET['page'] ) && $_GET['page'] == 'wfacp' ) {
				remove_action( 'init', 'et_add_divi_support_center' );
			}
		} );

		add_action( 'init', [ $this, 'remove_actions' ], 0 );
		add_action( 'wfacp_internal_css', [ $this, 'internal_css' ] );
		/* Disabled override the body layout on AeroCheckout  */
		add_action( 'wfacp_after_template_found', array( $this, 'maybe_disable_theme_builder' ), 8 );
	}

	public function remove_actions() {
		if ( wfacp_elementor_edit_mode() ) {
			remove_action( 'init', 'et_sync_custom_css_options' );
		}
	}

	public function maybe_disable_theme_builder() {

		if ( ! function_exists( 'et_setup_theme' ) || ! defined( 'ET_THEME_BUILDER_BODY_LAYOUT_POST_TYPE' ) ) {
			return;
		}
		$is_global_checkout = WFACP_Core()->public->is_checkout_override();
		if ( $is_global_checkout == true ) {
			$template_array = [
				'wfacp-canvas.php',
				'template-default-boxed.php',
				'page-template-blank.php'
			];
			$disable_for    = apply_filters( 'et_builder_compatibility_wfacp_checkout_templates_without_theme_builder', $template_array );
			$template       = get_post_meta( get_the_ID(), '_wp_page_template', true );
			if ( in_array( $template, $disable_for, true ) ) {
				add_filter( 'et_theme_builder_template_layouts', array( $this, 'disable_theme_builder' ) );
			}
		}

	}

	public function disable_theme_builder( $layouts ) {
		if ( isset( $layouts[ ET_THEME_BUILDER_BODY_LAYOUT_POST_TYPE ]['override'] ) ) {
			$layouts[ ET_THEME_BUILDER_BODY_LAYOUT_POST_TYPE ]['override'] = false;
		}

		return $layouts;
	}

	public function internal_css() {

		if ( ! defined( 'ET_CORE_VERSION' ) || ! function_exists( 'et_setup_theme' ) ) {
			return;
		}
		?>

        <style>
            #wfacp-e-form .wfacp-form .woocommerce-form-login-toggle .woocommerce-info a.showlogin,
            #wfacp-e-form .wfacp-form .woocommerce-form-login-toggle .woocommerce-info a {
                color: #dd7575 !important;;

            }

            #wfacp-e-form .wfacp_main_form .woocommerce-form-login-toggle .woocommerce-info a:hover,
            #wfacp-e-form .wfacp_main_form a span:hover,
            #wfacp-e-form .wfacp_main_form label a:not(.woocommerce-terms-and-conditions-link):hover,
            #wfacp-e-form .wfacp_main_form table tr td a:hover, body:not(.wfacpef_page)
            #wfacp-e-form .wfacp_main_form a:not(.wfacp_breadcrumb_link):hover,
            body:not(.wfacpef_page) #wfacp-e-form .wfacp_main_form ul li a:not(.wfacp_breadcrumb_link):hover {
                color: #965d5d !important;
            }
        </style>
		<?php
	}

}

WFACP_Plugin_Compatibilities::register( new WFACP_Compatibility_With_Divi(), 'divi' );
